import { Body, Controller, Get, Param, Post, UseGuards } from "@nestjs/common";
import { AuthGuard } from "@nestjs/passport";
import { ROLES } from "src/constant/roles.contant";
import { IsSuccessResponseDTO } from "src/dto/1.0.0/general/is-success-response.dto";
import { AllVrStatisticsResponseDTO } from "src/dto/1.0.0/statistics/all-vr-statistics-response.dto";
import { CourseExamStatisticsResponseDTO } from "src/dto/1.0.0/statistics/course-exam-response.dto";
import { UserStatisticsSaveDTO } from "src/dto/1.0.0/statistics/user-satistics-save";
import { UserCourseTactTmeSaveDTO } from "src/dto/1.0.0/statistics/user-tact-time-save.dto";
import { TactStatistics } from "src/entity/statistic-tact.entity";
import { TactAction } from "src/entity/tact-action.entity";
import { User } from "src/entity/user.entity";
import { Roles } from "src/module/auth/decorator/roles.decorator";
import { CoursesService100 } from "src/service/1.0.0/course.service";
import { StatisticsService100 } from "src/service/1.0.0/statistic.service";
import { TactStatisticService100 } from "src/service/1.0.0/tact-statistic.service";

@Controller("1.0.0/statistics")
export class StatisticsController100 {
  constructor(
    private readonly courseService: CoursesService100,
    private readonly statisticsService: StatisticsService100,
    private readonly tactStatisticsService: TactStatisticService100,
  ) { }

  @Get("get-course-exam-statistics/:id")
  async downloadflag(
    @Param("id") id: number
  ): Promise<CourseExamStatisticsResponseDTO> {
    return await this.courseService.getExamStatistics(id);
  }

  @UseGuards(AuthGuard("jwt"))
  @Post("upload-vr-statistics")
  async uploadVrStatistics(
    @Body() params: UserStatisticsSaveDTO
  ): Promise<IsSuccessResponseDTO> {
    return await this.statisticsService.uploadStatistics(params);
  }

  // @UseGuards(AuthGuard("jwt"))
  // @Roles([ROLES.admin, ROLES.maintainer])
  @Get("all-vr-statistics")
  async getAllVrStatistics(
  ): Promise<AllVrStatisticsResponseDTO> {
    return await this.statisticsService.getAllVrStatistics();
  }

  //TACT STATISCTICS
  @UseGuards(AuthGuard("jwt"))
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-tact-statistics")
  async getAllTactStatistics(
  ): Promise<TactStatistics[]> {
    return await this.tactStatisticsService.getAll();
  }
  @UseGuards(AuthGuard("jwt"))
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-tact-statistics-by-course/:id")
  async getAllTactStatisticsByCourseId(
    @Param("id") id: Number
  ): Promise<TactStatistics[]> {
    return await this.tactStatisticsService.getAllByCourseId(id);
  }

  @UseGuards(AuthGuard("jwt"))
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-tact-statistics-action-by-tact-statistics/:id")
  async getAllTactStatisticsActionByTactStatisticsId(
    @Param("id") id: Number
  ): Promise<TactAction[]> {
    return await this.tactStatisticsService.getAllTactStatisticsActionByTactStatisticsId(id);
  }

  @UseGuards(AuthGuard("jwt"))
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-tact-statistics-by-user/:id")
  async getAllTactStatisticsByUserId(
    @Param("id") id: Number
  ): Promise<TactStatistics[]> {
    return await this.tactStatisticsService.getAllByUserId(id);
  }
  @UseGuards(AuthGuard("jwt"))
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-tact-statistics-by-user-and-course/:userId/:courseId")
  async getAllTactStatisticsByUserIdAndCourseId(
    @Param("userId") userId: Number,
    @Param("courseId") courseId: Number,
  ): Promise<TactStatistics[]> {
    return await this.tactStatisticsService.getAllTactStatisticsByUserIdAndCourseId(userId, courseId);
  }

  @UseGuards(AuthGuard("jwt"))
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-tact-statistics-users-by-course/:id")
  async getAllTactStatisticsUsersByCourseID(
    @Param("id") id: Number
  ): Promise<User[]> {
    return await this.tactStatisticsService.getAllTactStatisticsUsersByCourseID(id);
  }


  @UseGuards(AuthGuard("jwt"))
  @Roles([ROLES.admin, ROLES.maintainer, ROLES.user])
  @Post("create-tact-statistics")
  async createTactStatistic(
    @Body() params: UserCourseTactTmeSaveDTO
  ): Promise<IsSuccessResponseDTO> {
    return await this.tactStatisticsService.createTactStatistic(params);
  }


}
