import {
  Controller,
  Get,
  UseGuards,
  Post,
  Body,
  Request,
  HttpCode,
  Param,
  Delete,
  Query,
} from "@nestjs/common";
import { AuthGuard } from "@nestjs/passport";
import { JwtService } from "@nestjs/jwt";
import { LoginDTO } from "../../dto/1.0.0/user/login.dto";
import { UserService100 } from "../../service/1.0.0/user.service";
import { User } from "../../entity/user.entity";
import { ModifyUserDTO } from "../../dto/1.0.0/user/modifyuser.dto";
import { RegistrationResponseDTO } from "../../dto/1.0.0/user/registration.response.dto";
import { LoginResponseDTO } from "../../dto/1.0.0/user/login.response.dto";
import { RolesGuard } from "src/module/auth/guard/roles.guard";
import { ROLES } from "src/constant/roles.contant";
import { RegistrationFromCMSDTO } from "src/dto/1.0.0/user/registration-from-cms.dto";
import { Roles } from "src/module/auth/decorator/roles.decorator";
import { QrCodeLoginResponseDTO } from "src/dto/1.0.0/user/qr-code-login-response.dto";
import { QrLoginDTO } from "src/dto/1.0.0/user/qr-login.dto";
import { UserCourseResponseDTO } from "src/dto/1.0.0/user/user-course-response.dto";
import { ModifyAdminDTO } from "src/dto/1.0.0/user/modify-admin.dto";
import { AddAdminOrTrainerDTO } from "src/dto/1.0.0/user/add-admin-or-trainer.dto";
import { EditUserDataFromCMSDTO } from "src/dto/1.0.0/user/edit-user-data-from-cms.dto";
import { IsSuccessResponseDTO } from "src/dto/1.0.0/general/is-success-response.dto";

@Controller("1.0.0/users")
export class UserController100 {
  constructor(
    private readonly userService: UserService100,
    private readonly jwtService: JwtService
  ) {}

  @UseGuards(AuthGuard("local"))
  @Post("login")
  @HttpCode(200)
  async login(
    @Request() req: Request,
    @Body() params: LoginDTO
  ): Promise<LoginResponseDTO> {
    const user = (req as any).user as User;

    return {
      user,
      accessToken: this.jwtService.sign({
        sub: user.id,
      }),
    };
  }

  @UseGuards(AuthGuard("jwt"))
  @Roles([ROLES.admin, ROLES.maintainer])
  @Delete("delete-user-by-id/:userId")
  async deactivateUser(@Param("userId") userId) {
    return await this.userService.deactiavateUser(userId);
  }

  // @Post("forgot-password")
  // async forgotPassword(@Body() params: ForgotPasswordDTO) {
  //   return await this.userService.forgotPassword(params.email);
  // }
  @UseGuards(AuthGuard("jwt"), RolesGuard)
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-without-admin")
  async getAllUserWithoutAdmin() {
    return await this.userService.getAllFilteredUserWithoutAdmin();
  }
  @UseGuards(AuthGuard("jwt"), RolesGuard)
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-player")
  async getAllPlayer() {
    return await this.userService.getAllPlayer();
  }
  @UseGuards(AuthGuard("jwt"), RolesGuard)
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-maintainer")
  async getAllMainTrainer() {
    return await this.userService.getAllMaintainer();
  }

  @UseGuards(AuthGuard("jwt"), RolesGuard)
  @Roles([ROLES.admin, ROLES.maintainer])
  @Get("get-all-admin")
  async getAllAdmin() {
    return await this.userService.getAllAdmin();
  }

  @UseGuards(AuthGuard("jwt"), RolesGuard)
  @Roles([ROLES.admin, ROLES.maintainer])
  @Post("registration-from-cms")
  async registrationFromCMSbyTM(
    @Body() params: RegistrationFromCMSDTO
  ): Promise<RegistrationResponseDTO> {
    const user: User = await this.userService.createNewUserFromCMSbyAdmin(
      params
    );
    return {
      user,
    };
  }

  @Post("delete-users-by-ids")
  @Roles([ROLES.admin])
  async deleteUserByIDs(@Body() params: any) {
    return await this.userService.deleteUsersByIds(params);
  }

  @Post("activate-deactivate-users-by-ids/:userId")
  @Roles([ROLES.admin])
  async deactivateAndActivateUsersById(@Param("userId") userId) {
    return await this.userService.deactivateAndActivateUsersById(userId);
  }

  @UseGuards(AuthGuard("jwt"))
  @Get("user-by-id")
  async getOneById(@Query("userId") userId) {
    return await this.userService.getOneUser(userId);
  }

  @UseGuards(AuthGuard("jwt"))
  @Post("modify-admin-and-trainer")
  async modifyAdminAndTrainer(
    @Body() params: ModifyAdminDTO
  ): Promise<RegistrationResponseDTO> {
    const user: User = await this.userService.modifyAdminAndTrainer(params);

    return {
      user,
    };
  }
  @UseGuards(AuthGuard("jwt"))
  @Post("modify-user")
  async modifyUser(
    @Body() params: ModifyUserDTO
  ): Promise<RegistrationResponseDTO> {
    const user: User = await this.userService.modifyUser(params);

    return {
      user,
    };
  }

  @Post("login-with-qr")
  async loginWithQR(
    @Body() params: QrLoginDTO
  ): Promise<QrCodeLoginResponseDTO> {
    return await this.userService.loginWithQrCode(params);
  }
  @Post("edit-user-from-cms")
  async editUserFromCMS(
    @Body() params: EditUserDataFromCMSDTO
  ): Promise<IsSuccessResponseDTO> {
    return await this.userService.editUserFromProfile(params);
  }
  //@UseGuards(AuthGuard("jwt"))
  @Get("users-by-course/:courseId")
  async getUserByCourse(
    @Param("courseId") courseId
  ): Promise<UserCourseResponseDTO> {
    return await this.userService.getUserByCourse(courseId);
  }
  @Get("users-by-course-exam/:courseId")
  async getUserByCourseEcxam(
    @Param("courseId") courseId
  ): Promise<UserCourseResponseDTO> {
    return await this.userService.getUserByCourseExam(courseId);
  }
  @Get("new-code")
  async getNewUserCode() {
    return await this.userService.generateID();
  }
  @Post("new-user")
  async createNewUser(@Body() params: any) {
    const obj = await this.userService.createNewUser(params);
    return obj;
  }
  @Post("edit-user")
  async editUser(@Body() params: any) {
    const obj = await this.userService.editUser(params);
    return obj;
  }
  @Post("new-admin-or-trainer")
  async createAdminOrTrainer(@Body() params: AddAdminOrTrainerDTO) {
    const obj = await this.userService.createAdminOrTrainer(params);
    return obj;
  }
  @Get("user-course/:userId")
  async getUserCourses(@Param("userId") userId): Promise<any> {
    return await this.userService.getUserCourses(userId);
  }
}


