import { Column, CreateDateColumn, Entity, ManyToMany, ManyToOne, OneToMany, PrimaryGeneratedColumn, UpdateDateColumn } from "typeorm";
import { CourseModulType } from "./course-modul-type.entity";
import { CourseStatus } from "./course-status.entity";
import { FilesCourse } from "./file-course.entity";
import { Language } from "./language.entity";
import { TactStatistics } from "./statistic-tact.entity";
import { Statistics } from "./statistic.entity";
import { UserCourse } from "./user-course.entity";
import { User } from "./user.entity";

@Entity()
export class Course {
  @PrimaryGeneratedColumn({
    unsigned: true,
  })
  id: number;

  @Column({
    nullable: true,
    length: 255,
  })
  name: string | null;

  @Column("text", { nullable: true })
  description: string | null;

  @Column("text", { nullable: true })
  data: string | null;

  @Column("text", { nullable: true })
  documentation: string | null;

  @Column("decimal", { nullable: true, precision: 16, scale: 6, default: 0 })
  dataSize: number | null;

  @Column({ nullable: true, default: 0 })
  finishedStudentNumber: number | null;

  @Column({ nullable: true })
  requiredStudentNumber: number | null;

  @Column({
    default: false,
  })
  isQuality: Boolean | null;

  @Column({
    default: true,
  })
  isActive: Boolean | null;

  @ManyToMany(() => User)
  users: User[];

  @CreateDateColumn()
  created: Date;

  @UpdateDateColumn()
  modified: Date;

  //   @OneToMany(
  //     (type) => Files,
  //     (files) => files.course,
  //     { onUpdate: "CASCADE", onDelete: "CASCADE" }
  //   )
  //   files: Files[] | null;

  //ONE TO MANY
  @OneToMany(
    (type) => UserCourse,
    (userCourse) => userCourse.course,
    { onDelete: "CASCADE", onUpdate: "CASCADE", cascade: true }
  )
  userCourse: UserCourse;

  @OneToMany(
    (type) => FilesCourse,
    (fileCourse) => fileCourse.course,
    { onDelete: "CASCADE", onUpdate: "CASCADE", cascade: true }
  )
  fileCourse: FilesCourse;

  //MANY TO ONE
  @ManyToOne(
    (type) => CourseStatus,
    (courseStatus) => courseStatus.course,
    { onDelete: "CASCADE", onUpdate: "CASCADE" }
  )
  courseStatus: CourseStatus;

  @ManyToOne(
    (type) => Language,
    (language) => language.courses,
    { onUpdate: "CASCADE", onDelete: "CASCADE" }
  )
  language: Language | null;

  @ManyToOne(
    (type) => CourseModulType,
    (courseModulType) => courseModulType.courses,
    { onDelete: "CASCADE", onUpdate: "CASCADE" }
  )
  courseModulType: CourseModulType;

  @ManyToOne(
    (type) => User,
    (owner) => owner.course,
    { onUpdate: "CASCADE", onDelete: "CASCADE" }
  )
  owner: User | null;
  @OneToMany(
    (type) => Statistics,
    (statistics) => statistics.course,
    { onDelete: "CASCADE", onUpdate: "CASCADE", cascade: true }
  )
  statistics: Statistics[];

  @OneToMany(
    (type) => TactStatistics,
    (statistics) => statistics.course,
    { onDelete: "CASCADE", onUpdate: "CASCADE", cascade: true }
  )
  tactStatistics: TactStatistics[];

  toJSON() {
    return {
      id: this.id,
      name: this.name,
      description: this.description,
      finishedStudentNumber: this.finishedStudentNumber,
      requiredStudentNumber: this.requiredStudentNumber,
      data: this.data,
      documentation: this.documentation,
      dataSize: this.dataSize,
      users: this.users,
      userCourse: this.userCourse,
      fileCourse: this.fileCourse,
      isQuality: this.isQuality,
      isActive: this.isActive,
      courseStatus: this.courseStatus,
      courseModulType: this.courseModulType,
      language: this.language,
      owner: this.owner,
      created: Math.round(this.created.getTime() / 1000),
      modified: Math.round(this.modified.getTime() / 1000),
    };
  }
}
