import { Column, CreateDateColumn, Entity, ManyToOne, OneToMany, PrimaryGeneratedColumn, UpdateDateColumn } from "typeorm";
import { CourseType } from "./course-type.entity";
import { Course } from "./course.entity";
import { User } from "./user.entity";

@Entity()
export class UserCourse {
  @PrimaryGeneratedColumn({
    unsigned: true,
  })
  id: number;

  @Column({
    nullable: true,
    length: 255,
  })
  name: string | null;
  @Column({
    nullable: true,
    length: 255,
  })
  qrCode: string | null;
  @Column({
    nullable: true,
    length: 255,
  })
  pdfName: string | null;
  @Column({
    default: true,
  })
  isActive: Boolean;
  @Column({
    default: false,
  })
  isFinished: Boolean;
  @Column({
    default: false,
  })
  examSuccessful: Boolean;

  @ManyToOne(
    (type) => User,
    (user) => user.userCourse,
    { onUpdate: "CASCADE", onDelete: "CASCADE" }
  )
  user: User | null;

  @Column({
    unsigned: true,
  })
  userId: number;

  @ManyToOne(
    (type) => Course,
    (course) => course.userCourse,
    { onUpdate: "CASCADE", onDelete: "CASCADE" }
  )
  course: Course | null;

  @Column({
    unsigned: true,
  })
  courseId: number;

  @ManyToOne(
    (type) => CourseType,
    (courseType) => courseType.userCourse,
    { onUpdate: "CASCADE", onDelete: "CASCADE" }
  )
  courseType: CourseType | null;

  @Column({
    unsigned: true,
  })
  courseTypeId: number;

  @Column({
    nullable: true,
  })
  courseStart: Date | null;

  @Column("decimal", { default: 0 })
  timeSpent: number | null;

  @Column({
    nullable: true,
  })
  examRating: number | null;

  @CreateDateColumn()
  created: Date;

  @UpdateDateColumn()
  modified: Date;
}
