import {
  Column,
  Entity,
  PrimaryGeneratedColumn,
  CreateDateColumn,
  UpdateDateColumn,
  BeforeInsert,
  BeforeUpdate,
  AfterLoad,
  ManyToOne,
  ManyToMany,
  JoinTable,
  OneToMany,
} from "typeorm";
import * as bcrypt from "bcryptjs";
import { Group } from "./group.entity";
import { Course } from "./course.entity";
import { UserUniqueFiled } from "./user-unique-field.entity";
import { UserCourse } from "./user-course.entity";
import { Statistics } from "./statistic.entity";
import { TactStatistics } from "./statistic-tact.entity";

@Entity()
export class User {
  @PrimaryGeneratedColumn({
    unsigned: true,
  })
  id: number;

  @Column({
    nullable: true,
    length: 255,
    unique: true,
    charset: "utf8",
    collation: "utf8_hungarian_ci",
  })
  email: string | null;

  @Column({
    nullable: true,
    length: 255,
    unique: true,
  })
  generatedID: string | null;

  @Column({
    nullable: true,
    length: 255,
    charset: "utf8",
    collation: "utf8_hungarian_ci",
  })
  firstName: string | null;
  @Column({
    nullable: true,
    length: 255,
    charset: "utf8",
    collation: "utf8_hungarian_ci",
  })
  lastName: string | null;

  @Column({
    nullable: true,
    length: 255,
    charset: "utf8",
    collation: "utf8_hungarian_ci",
  })
  password: string | null;

  @Column({
    nullable: true,
    length: 255,
    charset: "utf8",
    collation: "utf8_hungarian_ci",
  })
  jobGroup: string | null;

  @Column({
    nullable: true,
  })
  workStartDate: Date | null;
  @Column({
    nullable: true,
    length: 255,
    charset: "utf8",
    collation: "utf8_hungarian_ci",
  })
  genderIdentity: string | null;

  @Column({
    nullable: true,
    length: 255,
    charset: "utf8",
    collation: "utf8_hungarian_ci",
  })
  birthName: string | null;

  @Column({
    nullable: true,
    length: 255,
    charset: "utf8",
    collation: "utf8_hungarian_ci",
  })
  rank: string | null;
  @Column({
    nullable: true,
    length: 255,
    charset: "utf8",
    collation: "utf8_hungarian_ci",
  })
  organizationalUnit: string | null;

  @Column({
    nullable: true,
  })
  birthDate: Date | null;

  @Column({
    nullable: false,
    default: true,
  })
  isActive: Boolean;

  @CreateDateColumn()
  created: Date;

  @UpdateDateColumn()
  modified: Date;

  @BeforeInsert()
  beforeInsert() {
    if (
      this.password !== undefined &&
      this.password !== null &&
      this.password !== ""
    ) {
      this.password = bcrypt.hashSync(this.password);
    }
  }

  @BeforeUpdate()
  beforeUpdate() {
    if (this.tempPassword !== this.password) {
      this.password = bcrypt.hashSync(this.password);
    }
  }
  private tempPassword: string;

  @AfterLoad()
  afterLoad(): void {
    this.tempPassword = this.password;
  }

  @ManyToOne(
    (type) => Group,
    (group) => group.users,
    { onUpdate: "CASCADE", onDelete: "CASCADE" }
  )
  group: Group | null;

  @Column({
    unsigned: true,
  })
  groupId: number;

  @ManyToMany(() => Course)
  @JoinTable()
  courses: Course[];

  @OneToMany(
    (type) => UserUniqueFiled,
    (userUniqueField) => userUniqueField.user,
    { onDelete: "CASCADE", onUpdate: "CASCADE", cascade: true }
  )
  userUniqueField: UserUniqueFiled;

  @OneToMany(
    (type) => Statistics,
    (statistics) => statistics.user,
    { onDelete: "CASCADE", onUpdate: "CASCADE", cascade: true }
  )
  statistics: Statistics[];

  @OneToMany(
    (type) => TactStatistics,
    (statistics) => statistics.user,
    { onDelete: "CASCADE", onUpdate: "CASCADE", cascade: true }
  )
  tactStatistics: TactStatistics[];

  @OneToMany(
    (type) => UserCourse,
    (userCourse) => userCourse.user,
    { onDelete: "CASCADE", onUpdate: "CASCADE", cascade: true }
  )
  userCourse: UserCourse;

  @OneToMany(
    (type) => Course,
    (course) => course.owner,
    { onDelete: "CASCADE", onUpdate: "CASCADE", cascade: true }
  )
  course: Course[];

  toJSON() {
    return {
      id: this.id,
      email: this.email,
      firstName: this.firstName,
      lastName: this.lastName,
      rank: this.rank,
      birthName: this.birthName,
      genderIdentity: this.genderIdentity,
      group: this.group,
      groupId: this.groupId,
      birthDate:
        this.birthDate !== null
          ? Math.round(this.birthDate.getTime() / 1000)
          : null,
      workStartDate:
        this.workStartDate !== null
          ? Math.round(this.workStartDate.getTime() / 1000)
          : null,
      jobGroup: this.jobGroup,
      isActive: this.isActive,
      courses: this.courses,
      organizationalUnit: this.organizationalUnit,
      generatedID: this.generatedID,
      userUniqueField: this.userUniqueField,
      created: Math.round(this.created.getTime() / 1000),
      modified: Math.round(this.modified.getTime() / 1000),
    };
  }
}