import { Config } from "./module/config/entity/config";
Config.populate();

import { NestFactory } from "@nestjs/core";
import { AppModule } from "./app.module";
import { ApiExceptionFilter } from "./module/routing/filter/api.exception.filter";
import { AllExceptionsFilter } from "./module/routing/filter/all.exceptions.filter";
import { ValidationError } from "class-validator";
import { InvalidParamsException } from "./module/routing/exception/invalid.params.exception";
import { ValidationPipe } from "@nestjs/common";
import { UnauthorizedExceptionFilter } from "./module/routing/filter/unauthorized.exception.filter";
import { ResetDefaultValuesInterceptor } from "./module/routing/interceptor/reset.default.values.interceptor";
import { ForbiddenExceptionFilter } from "./module/routing/filter/forbidden.exception.filter";

const envPort: number = parseInt(process.env.NODE_PORT, 10);
const port: number =
  !isNaN(envPort) && envPort > 1 && envPort < 65535 ? envPort : 80;

async function bootstrap() {
  const app = await NestFactory.create(AppModule);

  if (Config.get("NODE_ENV") === "development") {
    app.enableCors();
  }
  app.useGlobalInterceptors(new ResetDefaultValuesInterceptor());

  app.useGlobalPipes(
    new ValidationPipe({
      exceptionFactory: (errors: ValidationError[]) => {
        throw new InvalidParamsException(errors);
      },
      whitelist: true,
      validationError: {
        target: false,
        value: false,
      },
    })
  );

  app.useGlobalFilters(
    new AllExceptionsFilter(),
    new ApiExceptionFilter(),
    new UnauthorizedExceptionFilter(),
    new ForbiddenExceptionFilter()
  );

  const server = await app.listen(port);
  server.setTimeout(1800000);
}

bootstrap();
