import { registerDecorator, ValidationOptions, ValidationArguments, validate } from 'class-validator';
import { plainToClass } from 'class-transformer';

export function IsInstancesOf(className, validationOptions?: ValidationOptions) {

    return function (object: Object, propertyName: string) {
        registerDecorator({
            name: "IsInstancesOf",
            target: object.constructor,
            propertyName: propertyName,
            constraints: [],
            options: validationOptions,
            async: true,
            validator: {
                async validate(value: any, args: ValidationArguments): Promise<boolean> {
                    const object = plainToClass(className, value);
                    const errors = await validate(object);

                    return !errors.length;
                }
            }
        });
    }
}