import { Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Course } from "src/entity/course.entity";
import { Repository } from "typeorm";
import * as fs from "fs";
import { User } from "src/entity/user.entity";
import { UserStatisticsSaveDTO } from "src/dto/1.0.0/statistics/user-satistics-save";
import { UserNotFoundException } from "src/exception/user/user-not-found.exception";
import { CourseNotFoundException } from "src/exception/course/course-not-found.exception";
import { Statistics } from "src/entity/statistic.entity";
import { StatisticsAction } from "src/entity/statistic-action.entity";
import { IsSuccessResponseDTO } from "src/dto/1.0.0/general/is-success-response.dto";
import { AllVrStatisticsResponseDTO } from "src/dto/1.0.0/statistics/all-vr-statistics-response.dto";

@Injectable()
export class StatisticsService100 {
    constructor(
        @InjectRepository(Course)
        private readonly coursespository: Repository<Course>,
        @InjectRepository(User)
        private readonly userpository: Repository<User>,
        @InjectRepository(Statistics)
        private readonly statisticspository: Repository<Statistics>,
        @InjectRepository(StatisticsAction)
        private readonly statisticsActionpository: Repository<StatisticsAction>,
    ) { }

    async uploadStatistics(params: UserStatisticsSaveDTO): Promise<IsSuccessResponseDTO> {
        const user = await this.userpository.findOne({ where: { id: params.userId } })
        const course = await this.coursespository.findOne({ where: { id: params.courseId } })
        if (!user) {
            throw new UserNotFoundException()
        }
        if (!course) {
            throw new CourseNotFoundException()
        }
        const newStatistics = this.statisticspository.create()
        newStatistics.course = course
        newStatistics.user = user
        newStatistics.date = params.date
        newStatistics.name = params.name
        newStatistics.success = params.success
        newStatistics.time = params.time
        newStatistics.type = params.type

        await this.statisticspository.save(newStatistics)

        for (let index = 0; index < params.actions.length; index++) {
            const actAction = params.actions[index];
            const newAction = this.statisticsActionpository.create()
            newAction.actionType = actAction.actionType
            newAction.name = actAction.name
            newAction.statistics = newStatistics
            newAction.timeRecognition = actAction.timeRecognition
            newAction.timeSpent = actAction.timeSpent
            newAction.type = actAction.type
            await this.statisticsActionpository.save(newAction)
        }
        return { success: true }
    }
    async getAllVrStatistics(): Promise<AllVrStatisticsResponseDTO> {

        return { vrStatistics: await this.statisticspository.find({ relations: ["user", "course", "statisticAction"] }) }
    }
}
