import { Injectable, NotFoundException } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { TactStatistics } from 'src/entity/statistic-tact.entity';
import { User } from 'src/entity/user.entity';
import { Course } from 'src/entity/course.entity';
import { CourseNotFoundException } from 'src/exception/course/course-not-found.exception';
import { UserNotFoundException } from 'src/exception/user/user-not-found.exception';
import { UserCourseTactTmeSaveDTO } from 'src/dto/1.0.0/statistics/user-tact-time-save.dto';
import { TactAction } from 'src/entity/tact-action.entity';
import { IsSuccessResponseDTO } from 'src/dto/1.0.0/general/is-success-response.dto';


@Injectable()
export class TactStatisticService100 {
    constructor(
        @InjectRepository(TactStatistics)
        private readonly tactStatisticsRepository: Repository<TactStatistics>,
        @InjectRepository(TactAction)
        private readonly tactActionRepository: Repository<TactAction>,
        @InjectRepository(User)
        private readonly userRepository: Repository<User>,
        @InjectRepository(Course)
        private readonly coursepository: Repository<Course>
    ) { }

    async getAll(): Promise<TactStatistics[]> {
        const tactStatistics = await this.tactStatisticsRepository.find({ relations: ['tactAction'] });
        return tactStatistics
    }

    async getAllByCourseId(courseId: Number): Promise<TactStatistics[]> {
        const course = await this.coursepository.findOne({ where: { id: courseId } })
        if (!course) {
            throw new CourseNotFoundException()
        }
        const tactStatistics = await this.tactStatisticsRepository.find({ where: { course: course }, relations: ['tactAction'] });
        return tactStatistics
    }
    async getAllTactStatisticsActionByTactStatisticsId(tactStatisticsId: Number): Promise<TactAction[]> {
        const tactStatistics = await this.tactStatisticsRepository.findOne({ where: { id: tactStatisticsId } })
        if (!tactStatistics) {
            throw new NotFoundException()
        }
        const tactStatisticsActions = await this.tactActionRepository.find({ where: { tactStatistics: tactStatistics } });
        return tactStatisticsActions
    }

    async getAllByUserId(userId: Number): Promise<TactStatistics[]> {
        const user = await this.userRepository.findOne({ where: { id: userId } })
        if (!user) {
            throw new UserNotFoundException()
        }
        const tactStatistics = await this.tactStatisticsRepository.find({ where: { user: user }, relations: ['tactAction'] });
        return tactStatistics
    }

    async getAllTactStatisticsByUserIdAndCourseId(userId: Number, courseId: Number): Promise<TactStatistics[]> {
        const user = await this.userRepository.findOne({ where: { id: userId } })
        const course = await this.coursepository.findOne({ where: { id: courseId } })
        if (!user) {
            throw new UserNotFoundException()
        }

        if (!course) {
            throw new CourseNotFoundException()
        }

        const tactStatistics = await this.tactStatisticsRepository.find({ where: { user: user, course: course }, relations: ['tactAction'] });
        return tactStatistics
    }

    async getAllTactStatisticsUsersByCourseID(courseId: Number): Promise<User[]> {
        const course = await this.coursepository.findOne({ where: { id: courseId } })
        if (!course) {
            throw new CourseNotFoundException()
        }
        const tactStatistics = await this.tactStatisticsRepository.find({ where: { course: course }, relations: ['tactAction', "user"] });
        const users: User[] = []
        for (let index = 0; index < tactStatistics.length; index++) {
            const actTactStatistics = tactStatistics[index];
            // console.log(actTactStatistics.user);
            // console.log(users.some(user => user.id === actTactStatistics.user.id))
            if (!users.some(user => user.id === actTactStatistics.user.id)) {
                users.push(actTactStatistics.user)
            }
        }

        return users
    }

    async createTactStatistic(params: UserCourseTactTmeSaveDTO): Promise<IsSuccessResponseDTO> {
        const course = await this.coursepository.findOne({ where: { id: params.courseId } })
        const user = await this.userRepository.findOne({ where: { id: params.userId } })

        if (!course) {
            throw new CourseNotFoundException()
        }

        if (!user) {
            throw new UserNotFoundException()
        }

        let newTactStatistics = this.tactStatisticsRepository.create()
        newTactStatistics.course = course
        newTactStatistics.user = user
        newTactStatistics = await this.tactStatisticsRepository.save(newTactStatistics)

        for (let index = 0; index < params.savedActions.length; index++) {
            const actSaveAction = params.savedActions[index];
            const newSaveAction = this.tactActionRepository.create()

            newSaveAction.actionComment = actSaveAction.actionComment
            newSaveAction.tactTimeCurrent = actSaveAction.tactTimeCurrent
            newSaveAction.tactTimeTarget = actSaveAction.tactTimeTarget
            newSaveAction.tactStatistics = newTactStatistics

            await this.tactActionRepository.save(newSaveAction)
        }

        return { success: true }
    }
}